: ${IPHONE_SDKVERSION:=`xcodebuild -showsdks | grep iphoneos | egrep "[[:digit:]]+\.[[:digit:]]+" -o | tail -1`}
: ${OSX_SDKVERSION:=10.9.2}
: ${XCODE_ROOT:=`xcode-select -print-path`}
: ${EXTRA_CPPFLAGS:="-std=c++11 -stdlib=libc++"}

: ${TARBALLDIR:=`pwd`}
: ${IOSBUILDDIR:=`pwd`/ios/build}
: ${OSXBUILDDIR:=`pwd`/osx/build}
: ${PREFIXDIR:=`pwd`/ios/prefix}
: ${IOSFRAMEWORKDIR:=`pwd`/ios/framework}
: ${OSXFRAMEWORKDIR:=`pwd`/osx/framework}
: ${COMPILER:="clang++"}
: ${PROJECT_NAME:=hashlibpp}

: ${VERSION:=0.3.4}

export CONFIGURATION=$2
export SOLUTIONDIR=$1
if [ $CONFIGURATION="" ]
then
    export CONFIGURATION="release"
fi
    
if [ $SOLUTIONDIR="" ]
    then
    export SOLUTIONDIR="bin"
fi

#: ${PLATFORM:=iPhoneOS}
#===============================================================================
ARM_DEV_CMD="xcrun --sdk iphoneos"
SIM_DEV_CMD="xcrun --sdk iphonesimulator"
OSX_DEV_CMD="xcrun --sdk macosx"

#===============================================================================

#################################
#包含库文件

: ${LIBS+=-lz}

: ${LIBS+=-L$(IPHONESDK)/usr/lib/}
: ${LIBS+=-L$(IPHONESDK)/usr/lib/system}


#################################
#包含组件
: ${DFLAGS=-isysroot ${IPHONESDK} ${INCPATH}}
: ${CFLAGS=${DFLAGS} -std=c++11 -stdlib=libstdc++}

#####################################################################################
#  Do not change any part of them unless you have understood this script very well  #
#  This is a kind remind.                                                           #
#####################################################################################

#===============================================================================
# Functions
#===============================================================================

abort()
{
    echo
    echo "Aborted: $@"
    exit 1
}

doneSection()
{
    echo
    echo "================================================================="
    echo "Done"
    echo
}

#===============================================================================

cleanEverythingReadyToStart()
{
    echo Cleaning everything before we start to build...

    rm -rf iphone-build iphonesim-build osx-build
    rm -rf "$IOSBUILDDIR"
    rm -rf "$OSXBUILDDIR"
    rm -rf "$PREFIXDIR"
    rm -rf "$IOSFRAMEWORKDIR/$FRAMEWORK_NAME.framework"
    rm -rf "$OSXFRAMEWORKDIR/$FRAMEWORK_NAME.framework"

    doneSection
}

#===============================================================================

#===============================================================================

buildAllLib()
{
    TARGET="$SOLUTIONDIR/$CONFIGURATION/lib${PROJECT_NAME}.a"
    mkdir -p "$SOLUTIONDIR/$CONFIGURATION" 
 
    rm -rf "$TARGET"
    $ARM_DEV_CMD lipo -create $IOSBUILDDIR/*/lib${PROJECT_NAME}.a $OSXBUILDDIR/*/lib${PROJECT_NAME}.a -output $TARGET
    doneSection
}

#===============================================================================
buildBoostForIPhoneOS()
{
   #: ${PLATFORM:=iPhoneOS}
   #: ${PLATFORM:=iPhoneSimulator}
   echo ......${PROJECT_NAME}
   export PLATFORM=$1
   
   if [ "$1" = "iPhoneOS" ] 
   then
       export BUILD_PATH=${IOSBUILDDIR}
   else
       export BUILD_PATH=${OSXBUILDDIR}
   fi

   if [ "$2" = "i386" ]
   then     
       export BUILD_PRX="${SIM_DEV_CMD}"
   elif [ "$2" = "x86_64" ]
    then
           #statements
       export BUILD_PRX="$SIM_DEV_CMD"
    else
       export BUILD_PRX="$ARM_DEV_CMD"
   fi

   export ARM_TYPE=$2
   export OBJ_DIR="${BUILD_PATH}/${ARM_TYPE}/obj"
   export PLAT_DEV_HOME=${XCODE_ROOT}/Platforms/${PLATFORM}.platform/Developer
   export SDK_ROOT=${PLAT_DEV_HOME}/SDKs/${PLATFORM}${IPHONE_SDKVERSION}.sdk
   export GCC_BIN=${XCODE_ROOT}/Toolchains/XcodeDefault.xctoolchain/usr/bin/${COMPILER}
   export CC=${XCODE_ROOT}/Toolchains/XcodeDefault.xctoolchain/usr/bin/${COMPILER}
   mkdir -p "${OBJ_DIR}"
 
   make PROJECT_NAME=${PROJECT_NAME}
   rm "${BUILD_PATH}/${ARM_TYPE}/lib${PROJECT_NAME}.a"
   echo ...${ARM_TYPE}
   #echo $IOSBUILDDIR/armv6
   (cd "${BUILD_PATH}/${ARM_TYPE}"; $BUILD_PRX ar crus lib${PROJECT_NAME}.a obj/*.o; )
   doneSection
}

#===============================================================================
buildFramework()
{
    : ${1:?}
    FRAMEWORKDIR=$1
    BUILDDIR=$2

    VERSION_TYPE=Alpha
    FRAMEWORK_NAME=${PROJECT_NAME}
    FRAMEWORK_VERSION=A

    FRAMEWORK_CURRENT_VERSION=$VERSION
    FRAMEWORK_COMPATIBILITY_VERSION=$VERSION

    FRAMEWORK_BUNDLE=$FRAMEWORKDIR/$FRAMEWORK_NAME.framework
    echo "Framework: Building $FRAMEWORK_BUNDLE from $BUILDDIR..."

    rm -rf $FRAMEWORK_BUNDLE

    echo "Framework: Setting up directories..."
    mkdir -p $FRAMEWORK_BUNDLE
    mkdir -p $FRAMEWORK_BUNDLE/Versions
    mkdir -p $FRAMEWORK_BUNDLE/Versions/$FRAMEWORK_VERSION
    mkdir -p $FRAMEWORK_BUNDLE/Versions/$FRAMEWORK_VERSION/Resources
    mkdir -p $FRAMEWORK_BUNDLE/Versions/$FRAMEWORK_VERSION/Headers
    mkdir -p $FRAMEWORK_BUNDLE/Versions/$FRAMEWORK_VERSION/Documentation

    echo "Framework: Creating symlinks..."
    ln -s $FRAMEWORK_VERSION               $FRAMEWORK_BUNDLE/Versions/Current
    ln -s Versions/Current/Headers         $FRAMEWORK_BUNDLE/Headers
    ln -s Versions/Current/Resources       $FRAMEWORK_BUNDLE/Resources
    ln -s Versions/Current/Documentation   $FRAMEWORK_BUNDLE/Documentation
    ln -s Versions/Current/$FRAMEWORK_NAME $FRAMEWORK_BUNDLE/$FRAMEWORK_NAME

    FRAMEWORK_INSTALL_NAME=$FRAMEWORK_BUNDLE/Versions/$FRAMEWORK_VERSION/$FRAMEWORK_NAME

    echo "Lipoing library into $FRAMEWORK_INSTALL_NAME..."
    $ARM_DEV_CMD lipo -create $BUILDDIR/*/lib${PROJECT_NAME}.a -o "$FRAMEWORK_INSTALL_NAME" || abort "Lipo $1 failed"
    #$ARM_DEV_CMD lipo -create $BUILDDIR/*/lib${PROJECT_NAME}.a -output "$FRAMEWORK_INSTALL_NAME" || abort "Lipo $1 failed"

    echo "Framework: Copying includes..."
    #cp -r $PREFIXDIR/include/boost/*  $FRAMEWORK_BUNDLE/Headers/

    echo "Framework: Creating plist..."
    cat > $FRAMEWORK_BUNDLE/Resources/Info.plist <<EOF
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
<key>CFBundleDevelopmentRegion</key>
<string>English</string>
<key>CFBundleExecutable</key>
<string>${FRAMEWORK_NAME}</string>
<key>CFBundleIdentifier</key>
<string>org.boost</string>
<key>CFBundleInfoDictionaryVersion</key>
<string>6.0</string>
<key>CFBundlePackageType</key>
<string>FMWK</string>
<key>CFBundleSignature</key>
<string>????</string>
<key>CFBundleVersion</key>
<string>${FRAMEWORK_CURRENT_VERSION}</string>
</dict>
</plist>
EOF

    doneSection
}
